@@ -248,17 +248,24 @@ def execute_model(fs, X_train, y_train, X_test, y_test):
     logging.info("Final Results: ")
     logging.info(score)
 
-    json_file_location = "foreshadow.json"
-    fs.to_json(json_file_location)
+    # TODO Temporarily turn off the serialization as this requires
+    #  additional change and plus we may not need it at all.
+    # json_file_location = "foreshadow.json"
+    # fs.to_json(json_file_location)
 
     pickled_fitted_pipeline_location = "foreshadow_fitted_pipeline.p"
     fs.pickle_fitted_pipeline(pickled_fitted_pipeline_location)
 
+    # logging.info(
+    #     "Serialized foreshadow pipeline has been saved to {} "
+    #     "and {}. Refer to docs to read and process.".format(
+    #         json_file_location, pickled_fitted_pipeline_location
+    #     )
+    # )
+
     logging.info(
-        "Serialized foreshadow pipeline has been saved to {} "
-        "and {}. Refer to docs to read and process.".format(
-            json_file_location, pickled_fitted_pipeline_location
-        )
+        "Serialized foreshadow pipeline has been saved to {}. Refer to docs "
+        "to read and process.".format(pickled_fitted_pipeline_location)
     )
 
 
