@@ -1,35 +1,29 @@
 """Core end-to-end pipeline, foreshadow."""
 
-import inspect
-import warnings
 from typing import List, NoReturn, Union
 
-from sklearn.model_selection._search import BaseSearchCV
+from sklearn.pipeline import Pipeline
 
 from foreshadow.base import BaseEstimator
 from foreshadow.cachemanager import CacheManager
+from foreshadow.concrete.internals.cleaners.customizable_base import (
+    CustomizableBaseCleaner,
+)
 from foreshadow.estimators.auto import AutoEstimator
 from foreshadow.estimators.estimator_wrapper import EstimatorWrapper
 from foreshadow.intents import IntentType
 from foreshadow.logging import logging
-from foreshadow.optimizers import ParamSpec, Tuner
-from foreshadow.pipeline import SerializablePipeline
 from foreshadow.preparer import DataPreparer
-from foreshadow.serializers import (
-    ConcreteSerializerMixin,
-    _make_deserializable,
-)
 from foreshadow.utils import (
     AcceptedKey,
     ConfigKey,
     Override,
     ProblemType,
     check_df,
-    get_transformer,
 )
 
 
-class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
+class Foreshadow(BaseEstimator):
     """An end-to-end pipeline to preprocess and tune a machine learning model.
 
     Example:
@@ -52,15 +46,7 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
 
     """
 
-    def __init__(
-        self,
-        X_preparer=None,
-        y_preparer=None,
-        estimator=None,
-        problem_type=None,
-        optimizer=None,
-        optimizer_kwargs=None,
-    ):
+    def __init__(self, estimator=None, problem_type=None):
         if problem_type not in [
             ProblemType.CLASSIFICATION,
             ProblemType.REGRESSION,
@@ -74,24 +60,17 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
                 )
             )
         self.problem_type = problem_type
-        self.X_preparer = X_preparer
-        self.y_preparer = y_preparer
-        self.estimator = estimator
-        self.optimizer = optimizer
-        self.optimizer_kwargs = (
-            {} if optimizer_kwargs is None else optimizer_kwargs
+        self._X_preparer = DataPreparer(cache_manager=CacheManager())
+        self._y_preprarer = DataPreparer(
+            cache_manager=CacheManager(),
+            y_var=True,
+            problem_type=self.problem_type,
         )
+        self.estimator = estimator
         self.pipeline = None
         self.data_columns = None
         self.has_fitted = False
 
-        if isinstance(self.estimator, AutoEstimator) and optimizer is not None:
-            warnings.warn(
-                "An automatic estimator cannot be used with an optimizer."
-                " Proceeding without use of optimizer"
-            )
-            self.optimizer = None
-
         if self.y_preparer is not None:
             self.estimator_wrapper = EstimatorWrapper(
                 self.estimator, self.y_preparer
@@ -115,21 +94,7 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
 
         .. # noqa: I201
         """
-        return self._X_preprocessor
-
-    @X_preparer.setter
-    def X_preparer(self, dp):
-        if dp is not None:
-            if dp is False:
-                self._X_preprocessor = None
-            elif isinstance(dp, DataPreparer):
-                self._X_preprocessor = dp
-            else:
-                raise ValueError(
-                    "Invalid value: '{}' " "passed as X_preparer".format(dp)
-                )
-        else:
-            self._X_preprocessor = DataPreparer(cache_manager=CacheManager())
+        return self._X_preparer
 
     @property
     def y_preparer(self):  # noqa
@@ -147,23 +112,7 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
 
         .. # noqa: I201
         """
-        return self._y_preprocessor
-
-    @y_preparer.setter
-    def y_preparer(self, yp):
-        if yp is not None:
-            if yp is False:
-                self._y_preprocessor = None
-            elif isinstance(yp, DataPreparer):
-                self._y_preprocessor = yp
-            else:
-                raise ValueError("Invalid value passed as y_preparer")
-        else:
-            self._y_preprocessor = DataPreparer(
-                cache_manager=CacheManager(),
-                y_var=True,
-                problem_type=self.problem_type,
-            )
+        return self._y_preprarer
 
     @property
     def estimator(self):  # noqa
@@ -198,31 +147,6 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
                 else True
             )
 
-    @property
-    def optimizer(self):  # noqa
-        """Optimizer class that will fit the model.
-
-        Performs a grid or random search algorithm on the parameter space from
-        the preprocessors and estimators in the pipeline
-
-        :getter: Returns optimizer class
-
-        :setter: Verifies Optimizer class, defaults to None
-
-        Returns:
-            the optimizer object
-
-        .. # noqa: I201
-        """
-        return self._optimizer
-
-    @optimizer.setter
-    def optimizer(self, o):
-        if o is None or (inspect.isclass(o) and issubclass(o, BaseSearchCV)):
-            self._optimizer = o
-        else:
-            raise ValueError("Invalid optimizer: '{}' passed.".format(o))
-
     def _reset(self):
         if hasattr(self, "pipeline"):
             del self.pipeline
@@ -246,44 +170,14 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
         y_df = check_df(y_df)
         self.data_columns = X_df.columns.values.tolist()
 
-        if self.X_preparer is not None:
-            self.pipeline = SerializablePipeline(
-                [
-                    ("X_preparer", self.X_preparer),
-                    ("estimator_wrapper", self.estimator_wrapper),
-                ]
-            )
-        else:
-            self.pipeline = SerializablePipeline(
-                [("estimator_wrapper", self.estimator_wrapper)]
-            )
-
-        if self.optimizer is not None:
-            self.pipeline.fit(X_df, y_df)
-            params = ParamSpec(self.pipeline, X_df, y_df)
-            self.opt_instance = self.optimizer(
-                estimator=self.pipeline,
-                param_distributions=params,
-                **{
-                    "iid": True,
-                    "scoring": "accuracy",
-                    "n_iter": 10,
-                    "return_train_score": True,
-                }
-            )
-            self.tuner = Tuner(self.pipeline, params, self.opt_instance)
-            self.tuner.fit(X_df, y_df)
-            self.pipeline = self.tuner.transform(self.pipeline)
-            # extract trained preprocessors
-            if self.X_preparer is not None:
-                self.X_preparer = self.pipeline.steps[0][1]
-            if self.y_preparer is not None:
-                self.y_preparer = self.opt_instance.best_estimator_.steps[1][
-                    1
-                ].preprocessor
-        else:
-            self.pipeline.fit(X_df, y_df)
+        self.pipeline = Pipeline(
+            [
+                ("X_preparer", self.X_preparer),
+                ("estimator_wrapper", self.estimator_wrapper),
+            ]
+        )
 
+        self.pipeline.fit(X_df, y_df)
         self.has_fitted = True
 
         return self
@@ -359,81 +253,6 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
         self._prepare_predict(data_df.columns)
         return self.pipeline.score(data_df, y_df, sample_weight)
 
-    def dict_serialize(self, deep=False):
-        """Serialize the init parameters of the foreshadow object.
-
-        Args:
-            deep (bool): If True, will return the parameters for this estimator
-                recursively
-
-        Returns:
-            dict: The initialization parameters of the foreshadow object.
-
-        """
-        serialized = super().dict_serialize(deep=False)
-        serialized["estimator"] = self._customize_serialized_estimator(
-            self.estimator
-        )
-        return serialized
-
-    @staticmethod
-    def _customize_serialized_estimator(estimator):
-        if isinstance(estimator, AutoEstimator):
-            """For third party automl estimator, the estimator_kwargs
-            have different format and structure. To reduce verbosity,
-            this field is removed from the serialized object.
-            """
-            serialized_estimator = estimator.serialize()
-            serialized_estimator.pop("estimator_kwargs")
-        else:
-            serialized_estimator = estimator.get_params()
-            serialized_estimator["_class"] = (
-                estimator.__module__ + "." + type(estimator).__name__
-            )
-            serialized_estimator["_method"] = "dict"
-
-        result = serialized_estimator
-        return result
-
-    @classmethod
-    def dict_deserialize(cls, data):
-        """Deserialize the dictionary form of a foreshadow object.
-
-        Args:
-            data: The dictionary to parse as foreshadow object is constructed.
-
-        Returns:
-            object: A re-constructed foreshadow object.
-
-        """
-        serialized_estimator = data.pop("estimator")
-        estimator = cls._reconstruct_estimator(serialized_estimator)
-
-        params = _make_deserializable(data)
-        data_columns = params.pop("data_columns")
-        params["estimator"] = estimator
-
-        ret_tf = cls(**params)
-        ret_tf.data_columns = data_columns
-        return ret_tf
-
-    @classmethod
-    def _reconstruct_estimator(cls, data):
-        estimator_type = data.pop("_class")
-        _ = data.pop("_method")
-
-        if estimator_type == AutoEstimator.__name__:
-            class_name = estimator_type
-            module_path = None
-        else:
-            class_name = estimator_type.split(".")[-1]
-            module_path = ".".join(estimator_type.split(".")[0:-1])
-
-        estimator_class = get_transformer(class_name, source_lib=module_path)
-        estimator = estimator_class()
-        estimator.set_params(**data)
-        return estimator
-
     def get_params(self, deep=True):
         """Get params for this object. See super.
 
@@ -475,7 +294,7 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
         # been processed will be visible.
         cache_manager = self.X_preparer.cache_manager
         if self._has_column_in_cache_manager(column_name):
-            return cache_manager["intent"][column_name]
+            return cache_manager[AcceptedKey.INTENT][column_name]
         else:
             logging.info(
                 "No intent exists for column {}. Either the column "
@@ -516,7 +335,7 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
             )
             return False
         cache_manager = self.X_preparer.cache_manager
-        return True if column in cache_manager["intent"] else False
+        return True if column in cache_manager[AcceptedKey.INTENT] else False
 
     def override_intent(self, column_name: str, intent: str) -> NoReturn:
         """Override the intent of a particular column.
@@ -543,10 +362,10 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
         ):
             raise ValueError("Invalid Column {}".format(column_name))
         # Update the intent
-        self.X_preparer.cache_manager["override"][
+        self.X_preparer.cache_manager[AcceptedKey.OVERRIDE][
             "_".join([Override.INTENT, column_name])
         ] = intent
-        self.X_preparer.cache_manager["intent"][column_name] = intent
+        self.X_preparer.cache_manager[AcceptedKey.INTENT][column_name] = intent
 
     def configure_multiprocessing(self, n_job: int = 1) -> NoReturn:
         """Configure the multiprocessing option.
@@ -559,16 +378,22 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
             ConfigKey.N_JOBS
         ] = n_job
 
-    def set_processed_data_export_path(self, data_path: str) -> NoReturn:
+    def set_processed_data_export_path(
+        self, data_path: str, is_train: bool
+    ) -> NoReturn:
         """Set path to export data before feeding the data to the estimator.
 
         Args:
             data_path: the data path string
+            is_train: whether this is for training data
 
         """
-        self.X_preparer.cache_manager["config"][
-            ConfigKey.PROCESSED_DATA_EXPORT_PATH
-        ] = data_path
+        key = (
+            ConfigKey.PROCESSED_TRAINING_DATA_EXPORT_PATH
+            if is_train
+            else ConfigKey.PROCESSED_TEST_DATA_EXPORT_PATH
+        )
+        self.X_preparer.cache_manager[AcceptedKey.CONFIG][key] = data_path
 
     def pickle_fitted_pipeline(self, path: str) -> NoReturn:
         """Pickle the foreshadow object with the best pipeline estimator.
@@ -593,7 +418,9 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
             self.estimator = self.estimator.estimator.fitted_pipeline_
             # updating the estimator above will not update the reference in
             # the pipeline instance as it still points to the old object.
-            self.pipeline.steps[1] = ("estimator", self.estimator)
+            # Accessing the estimator wrapper object and update the
+            # estimator field.
+            self.pipeline.steps[1][1].estimator = self.estimator
 
         with open(path, "wb") as fopen:
             pickle.dump(self.pipeline, fopen)
@@ -623,3 +450,27 @@ class Foreshadow(BaseEstimator, ConcreteSerializerMixin):
         self.X_preparer.cache_manager[AcceptedKey.CONFIG][
             ConfigKey.SAMPLING_WITH_REPLACEMENT
         ] = replace
+
+    def register_customized_data_cleaner(
+        self, data_cleaners: List
+    ) -> NoReturn:
+        """**EXPERIMENTAL** Allow user to register a customized data cleaner.
+
+        Args:
+            data_cleaners: customized data cleaners
+
+        Raises:
+            ValueError: data cleaner must be a child class of the base cleaner.
+
+        """
+        for cleaner in data_cleaners:
+            if not issubclass(cleaner, CustomizableBaseCleaner):
+                raise ValueError(
+                    "cleaner {} must be a child class of the {} class.".format(
+                        str(cleaner), CustomizableBaseCleaner.__name__
+                    )
+                )
+
+        self.X_preparer.cache_manager[AcceptedKey.CUSTOMIZED_TRANSFORMERS][
+            ConfigKey.CUSTOMIZED_CLEANERS
+        ] = data_cleaners
