@@ -2,20 +2,20 @@
 
 from .cleaner import CleanerMapper
 from .data_exporter import DataExporterMapper
-from .feature_engineerer import FeatureEngineererMapper
-from .feature_reducer import FeatureReducerMapper
 from .feature_summarizer import FeatureSummarizerMapper
+from .flattener import FlattenMapper
 from .mapper import IntentMapper
 from .preparerstep import PreparerStep
 from .preprocessor import Preprocessor
 
 
 __all__ = [
+    "FlattenMapper",
     "CleanerMapper",
     "IntentMapper",
     "Preprocessor",
-    "FeatureEngineererMapper",
-    "FeatureReducerMapper",
+    # "FeatureEngineererMapper",
+    # "FeatureReducerMapper",
     "FeatureSummarizerMapper",
     "PreparerStep",
     "DataExporterMapper",
