@@ -1,8 +1,10 @@
 """Test the Foreshadow class."""
 
 import pytest
+from sklearn.model_selection import train_test_split
 
 from foreshadow.foreshadow import Foreshadow
+from foreshadow.intents import IntentType
 from foreshadow.utils import AcceptedKey, ProblemType
 from foreshadow.utils.testing import get_file_path
 
@@ -31,75 +33,12 @@ def test_foreshadow_defaults():
         and isinstance(foreshadow.y_preparer, DataPreparer)
         and isinstance(foreshadow.estimator_wrapper, EstimatorWrapper)
         and isinstance(foreshadow.estimator, AutoEstimator)
-        and foreshadow.optimizer is None
+        # and foreshadow.optimizer is None
         and foreshadow.pipeline is None
         and foreshadow.data_columns is None
     )
 
 
-def test_foreshadow_X_preparer_false():
-    from foreshadow.foreshadow import Foreshadow
-
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION, X_preparer=False
-    )
-    assert foreshadow.X_preparer is None
-
-
-def test_foreshadow_X_preparer_custom():
-    from foreshadow.foreshadow import Foreshadow
-    from foreshadow.preparer import DataPreparer
-    from foreshadow.cachemanager import CacheManager
-
-    dp = DataPreparer(cache_manager=CacheManager())
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION, X_preparer=dp
-    )
-    assert foreshadow.X_preparer == dp
-
-
-def test_foreshadow_X_preparer_error():
-    from foreshadow.foreshadow import Foreshadow
-
-    preprocessor = "Invalid"
-    with pytest.raises(ValueError) as e:
-        _ = Foreshadow(
-            problem_type=ProblemType.CLASSIFICATION, X_preparer=preprocessor
-        )
-
-    assert str(e.value) == "Invalid value: 'Invalid' passed as X_preparer"
-
-
-def test_foreshadow_y_preparer_false():
-    from foreshadow.foreshadow import Foreshadow
-
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION, y_preparer=False
-    )
-    assert foreshadow.y_preparer is None
-
-
-def test_foreshadow_y_preparer_custom():
-    from foreshadow.foreshadow import Foreshadow
-    from foreshadow.preparer import DataPreparer
-
-    dp = DataPreparer()
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION, y_preparer=dp
-    )
-    assert type(foreshadow.y_preparer) == DataPreparer
-
-
-def test_foreshadow_y_preparer_error():
-    from foreshadow.foreshadow import Foreshadow
-
-    dp = "Invalid"
-    with pytest.raises(ValueError) as e:
-        _ = Foreshadow(problem_type=ProblemType.CLASSIFICATION, y_preparer=dp)
-
-    assert str(e.value) == "Invalid value passed as y_preparer"
-
-
 def test_foreshadow_estimator_custom():
     from foreshadow.foreshadow import Foreshadow
     from foreshadow.base import BaseEstimator
@@ -123,122 +62,6 @@ def test_foreshadow_estimator_error():
     assert str(e.value) == "Invalid value passed as estimator"
 
 
-def test_foreshadow_optimizer_custom():
-    from foreshadow.foreshadow import Foreshadow
-    from sklearn.model_selection._search import BaseSearchCV
-    from foreshadow.base import BaseEstimator
-
-    class DummySearch(BaseSearchCV):
-        pass
-
-    # Need custom estimator to avoid warning
-    estimator = BaseEstimator()
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION,
-        estimator=estimator,
-        optimizer=DummySearch,
-    )
-    assert issubclass(foreshadow.optimizer, BaseSearchCV)
-
-
-def test_foreshadow_optimizer_error_invalid():
-    from foreshadow.foreshadow import Foreshadow
-
-    optimizer = "Invalid"
-    with pytest.raises(ValueError) as e:
-        _ = Foreshadow(
-            problem_type=ProblemType.CLASSIFICATION, optimizer=optimizer
-        )
-
-    assert str(e.value) == "Invalid optimizer: 'Invalid' passed."
-
-
-def test_foreshadow_optimizer_error_wrongclass():
-    from foreshadow.foreshadow import Foreshadow
-
-    optimizer = Foreshadow
-    with pytest.raises(ValueError) as e:
-        _ = Foreshadow(
-            problem_type=ProblemType.CLASSIFICATION, optimizer=optimizer
-        )
-
-    assert (
-        str(e.value) == "Invalid optimizer: '<class "
-        "'foreshadow.foreshadow.Foreshadow'>' passed."
-    )
-
-
-def test_foreshadow_warns_on_set_estimator_optimizer():
-    from foreshadow.foreshadow import Foreshadow
-    from sklearn.model_selection._search import BaseSearchCV
-
-    class DummySearch(BaseSearchCV):
-        pass
-
-    with pytest.warns(Warning) as w:
-        _ = Foreshadow(
-            problem_type=ProblemType.CLASSIFICATION, optimizer=DummySearch
-        )
-
-    assert str(w[0].message) == (
-        "An automatic estimator cannot be used with an"
-        " optimizer. Proceeding without use of optimizer"
-    )
-
-
-def test_foreshadow_custom_fit_estimate(mocker):
-    import numpy as np
-    from sklearn.pipeline import Pipeline
-    from sklearn.linear_model import LogisticRegression
-    from sklearn.preprocessing import OneHotEncoder
-    from sklearn.model_selection import train_test_split
-    from foreshadow.foreshadow import Foreshadow
-
-    np.random.seed(0)
-
-    X_pipeline = Pipeline([("xohe", OneHotEncoder())])
-    setattr(X_pipeline, "pipeline", X_pipeline)
-    estimator = LogisticRegression()
-
-    X = np.array([0] * 50 + [1] * 50).reshape((-1, 1))
-    y = np.array([0] * 50 + [1] * 50)
-    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.1)
-
-    # Let foreshadow set to defaults, we will overwrite them
-    X_preparer = mocker.PropertyMock(return_value=X_pipeline)
-    mocker.patch.object(Foreshadow, "X_preparer", X_preparer)
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.CLASSIFICATION,
-        y_preparer=False,
-        estimator=estimator,
-    )
-
-    foreshadow.fit(X_train, y_train)
-    foreshadow_predict = foreshadow.predict(X_test)
-    foreshadow_predict_proba = foreshadow.predict_proba(X_test)
-    foreshadow_score = foreshadow.score(X_test, y_test)
-    expected_predict = np.array([0, 1, 0, 1, 1, 1, 0, 1, 1, 1])
-    expected_predict_proba = np.array(
-        [
-            [0.9414791454949417, 0.05852085450505827],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.9414791454949417, 0.05852085450505827],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.9414791454949417, 0.05852085450505827],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.06331066362121573, 0.9366893363787843],
-            [0.06331066362121573, 0.9366893363787843],
-        ]
-    )
-    expected_score = 1.0
-
-    assert np.allclose(foreshadow_predict, expected_predict)
-    assert np.allclose(foreshadow_predict_proba, expected_predict_proba)
-    assert np.allclose(foreshadow_score, expected_score)
-
-
 def test_foreshadow_y_preparer(mocker):
     import numpy as np
     from sklearn.pipeline import Pipeline
@@ -266,9 +89,7 @@ def test_foreshadow_y_preparer(mocker):
     y_preparer = mocker.PropertyMock(return_value=y_pipeline)
     mocker.patch.object(Foreshadow, "y_preparer", y_preparer)
     foreshadow = Foreshadow(
-        problem_type=ProblemType.REGRESSION,
-        y_preparer=False,
-        estimator=estimator,
+        problem_type=ProblemType.REGRESSION, estimator=estimator
     )
     foreshadow.fit(X_train, y_train)
     foreshadow_predict = foreshadow.predict(X_test)
@@ -293,54 +114,6 @@ def test_foreshadow_y_preparer(mocker):
     assert np.allclose(foreshadow_score, expected_score)
 
 
-def test_foreshadow_without_x_processor():
-    import numpy as np
-    from sklearn.linear_model import LinearRegression
-    from sklearn.model_selection import train_test_split
-    from foreshadow.foreshadow import Foreshadow
-
-    np.random.seed(0)
-    estimator = LinearRegression()
-    X = np.arange(200).reshape((-1, 1))
-    y = np.random.normal(0, 1, 200).reshape((-1, 1))
-    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.1)
-
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.REGRESSION,
-        X_preparer=False,
-        y_preparer=False,
-        estimator=estimator,
-    )
-    foreshadow.fit(X_train, y_train)
-    foreshadow_predict = foreshadow.predict(X_test)
-    expected_predict = np.array(
-        [
-            [0.21789584803659176],
-            [-0.11658780412675052],
-            [-0.008315639264628194],
-            [-0.04698426957252905],
-            [0.229496437128962],
-            [0.09608966256670409],
-            [-0.07791917381884972],
-            [0.21402898500580167],
-            [0.20629525894422152],
-            [-0.023783091387788502],
-            [0.06902162135117351],
-            [0.10769025165907437],
-            [0.0825556419589388],
-            [0.16569319712092562],
-            [-0.11852123564214556],
-            [-0.11078750958056544],
-            [0.028419559527877614],
-            [-0.01604936532620832],
-            [0.03035299104327266],
-            [0.09222279953591403],
-        ]
-    )
-
-    assert np.allclose(foreshadow_predict, expected_predict)
-
-
 def test_foreshadow_predict_before_fit():
     import numpy as np
     from sklearn.linear_model import LinearRegression
@@ -354,10 +127,7 @@ def test_foreshadow_predict_before_fit():
     X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.1)
 
     foreshadow = Foreshadow(
-        problem_type=ProblemType.REGRESSION,
-        X_preparer=False,
-        y_preparer=False,
-        estimator=estimator,
+        problem_type=ProblemType.REGRESSION, estimator=estimator
     )
 
     with pytest.raises(ValueError) as e:
@@ -366,34 +136,6 @@ def test_foreshadow_predict_before_fit():
     assert str(e.value) == "Foreshadow has not been fit yet"
 
 
-def test_foreshadow_predict_diff_cols():
-    import numpy as np
-    from sklearn.linear_model import LinearRegression
-    from sklearn.model_selection import train_test_split
-    from foreshadow.foreshadow import Foreshadow
-
-    np.random.seed(0)
-    estimator = LinearRegression()
-    X = np.arange(200).reshape((-1, 2))
-    y = np.random.normal(0, 1, 100).reshape((-1, 1))
-    X_train, X_test, y_train, y_test = train_test_split(X, y, test_size=0.1)
-
-    foreshadow = Foreshadow(
-        problem_type=ProblemType.REGRESSION,
-        X_preparer=False,
-        y_preparer=False,
-        estimator=estimator,
-    )
-    foreshadow.fit(X_train, y_train)
-
-    with pytest.raises(ValueError) as e:
-        _ = foreshadow.predict(X_test[:, :-1])
-
-    assert (
-        str(e.value) == "Predict must have the same columns as train columns"
-    )
-
-
 @pytest.mark.skip("borken until parameter optimization is implemented")
 def test_foreshadow_param_optimize_fit(mocker):
     import pandas as pd
@@ -714,6 +456,7 @@ def test_core_foreshadow_example_classification():
         estimator=LogisticRegression(), problem_type=ProblemType.CLASSIFICATION
     )
     model.fit(X_train, y_train)
+
     score = f1_score(y_test, model.predict(X_test), average="weighted")
     print("Iris score: %f" % score)
 
@@ -741,14 +484,7 @@ def test_foreshadow_get_params_keys(deep):
     fs = Foreshadow(problem_type=ProblemType.CLASSIFICATION)
     params = fs.get_params(deep=deep)
 
-    desired_keys = [
-        "problem_type",
-        "X_preparer",
-        "estimator",
-        "y_preparer",
-        "optimizer",
-        "data_columns",
-    ]
+    desired_keys = ["problem_type", "estimator", "data_columns"]
     for key in desired_keys:
         assert key in params
 
@@ -789,7 +525,9 @@ def test_foreshadow_serialization_adults_small_classification_override():
 
     np.random.seed(1337)
 
-    adult = pd.read_csv("examples/adult_small.csv")
+    data_path = get_file_path("data", "adult_small.csv")
+
+    adult = pd.read_csv(data_path)
     X_df = adult.loc[:, "age":"workclass"]
     y_df = adult.loc[:, "class"]
 
@@ -824,7 +562,9 @@ def test_foreshadow_adults_small_classification_override_upfront():
 
     np.random.seed(1337)
 
-    adult = pd.read_csv("examples/adult_small.csv")
+    data_path = get_file_path("data", "adult_small.csv")
+
+    adult = pd.read_csv(data_path)
     X_df = adult.loc[:, "age":"workclass"]
     y_df = adult.loc[:, "class"]
 
@@ -912,27 +652,9 @@ def construct_foreshadow_object_common(
     return shadow
 
 
-@slow
-def test_foreshadow_serialization_adults_small_classification():
+def test_foreshadow_adults_classification():
     X_train, X_test, y_train, y_test = train_test_split_local_file_common(
-        file_path="examples/adult_small.csv",
-        X_start="age",
-        X_end="workclass",
-        target="class",
-    )
-    shadow = construct_foreshadow_object_common(
-        problem_type=ProblemType.CLASSIFICATION
-    )
-
-    shadow.fit(X_train, y_train)
-    score1 = shadow.score(X_test, y_test)
-    print(score1)
-
-
-@slow
-def test_foreshadow_serialization_adults_classification():
-    X_train, X_test, y_train, y_test = train_test_split_local_file_common(
-        file_path="examples/adult.csv",
+        file_path=get_file_path("data", "adult.csv"),
         X_start="age",
         X_end="native-country",
         target="class",
@@ -1007,7 +729,6 @@ def test_foreshadow_pickling_and_unpickling_non_tpot(tmpdir):
     assertions.assertAlmostEqual(score1, score2, places=2)
 
 
-@slow
 def test_foreshadow_pickling_and_unpickling_tpot(tmpdir):
     from foreshadow.foreshadow import Foreshadow
     import pandas as pd
@@ -1100,10 +821,9 @@ def test_foreshadow_configure_sampling():
     )
 
 
-@slow
 def test_foreshadow_sampling_performance_comparison():
     X_train, X_test, y_train, y_test = train_test_split_local_file_common(
-        file_path="examples/adult_small.csv",
+        file_path=get_file_path("data", "adult_small.csv"),
         X_start="age",
         X_end="workclass",
         target="class",
@@ -1155,8 +875,9 @@ def test_foreshadow_abort_on_empty_data_frame_after_cleaning(
 
     np.random.seed(1337)
 
-    local_file_folder = "examples"
-    data = pd.read_csv("/".join([local_file_folder, filename]))
+    data_path = get_file_path("data", filename)
+
+    data = pd.read_csv(data_path)
     X_df = data.loc[:, X_start:X_end]
     y_df = data.loc[:, target]
 
@@ -1205,8 +926,80 @@ def test_foreshadow_integration_data_cleaner_can_drop(
 
     np.random.seed(1337)
 
-    local_file_folder = "examples"
-    data = pd.read_csv("/".join([local_file_folder, filename]))
+    data_path = get_file_path("data", filename)
+
+    data = pd.read_csv(data_path)
+    # local_file_folder = "examples"
+    # data = pd.read_csv("/".join([local_file_folder, filename]))
+
+    X_df = data.loc[:, X_start:X_end]
+    y_df = data.loc[:, target]
+
+    X_train, X_test, y_train, y_test = train_test_split(
+        X_df, y_df, test_size=0.2
+    )
+
+    from foreshadow.estimators import AutoEstimator
+
+    estimator = AutoEstimator(
+        problem_type=problem_type,
+        auto="tpot",
+        estimator_kwargs={"max_time_mins": 1},
+    )
+
+    shadow = Foreshadow(estimator=estimator, problem_type=problem_type)
+
+    pickled_fitted_pipeline_location = tmpdir.join("fitted_pipeline.p")
+    shadow.fit(X_train, y_train)
+    shadow.pickle_fitted_pipeline(pickled_fitted_pipeline_location)
+
+    import pickle
+
+    with open(pickled_fitted_pipeline_location, "rb") as fopen:
+        pipeline = pickle.load(fopen)
+
+    # If there are new empty columns in the test set, the program should
+    # not fail.
+    X_test[X_start] = np.nan
+    score1 = shadow.score(X_test, y_test)
+    score2 = pipeline.score(X_test, y_test)
+
+    import unittest
+
+    assertions = unittest.TestCase("__init__")
+    # given the randomness of the tpot algorithm and the short run
+    # time we configured, there is no guarantee the performance can
+    # converge. The test here aims to evaluate if both cases have
+    # produced a reasonable score and the difference is small.
+    # assert score1 > 0.76 and score2 > 0.76
+    assertions.assertAlmostEqual(score1, score2, places=2)
+
+
+@pytest.mark.parametrize(
+    "filename,problem_type,X_start, X_end, target",
+    [
+        (
+            "adult_small.csv",
+            ProblemType.CLASSIFICATION,
+            "age",
+            "workclass",
+            "class",
+        )
+    ],
+)
+def test_foreshadow_integration_adult_small_piclking_unpickling(
+    filename, problem_type, X_start, X_end, target, tmpdir
+):
+    from foreshadow.foreshadow import Foreshadow
+    import pandas as pd
+    import numpy as np
+    from sklearn.model_selection import train_test_split
+
+    np.random.seed(1337)
+
+    data_path = get_file_path("data", filename)
+
+    data = pd.read_csv(data_path)
     X_df = data.loc[:, X_start:X_end]
     y_df = data.loc[:, target]
 
@@ -1245,3 +1038,243 @@ def test_foreshadow_integration_data_cleaner_can_drop(
     # produced a reasonable score and the difference is small.
     # assert score1 > 0.76 and score2 > 0.76
     assertions.assertAlmostEqual(score1, score2, places=2)
+
+
+def test_foreshadow_adults_small_user_provided_cleaner():
+    from foreshadow.foreshadow import Foreshadow
+    import pandas as pd
+    import numpy as np
+    from sklearn.model_selection import train_test_split
+    from sklearn.linear_model import LogisticRegression
+
+    np.random.seed(1337)
+
+    data_path = get_file_path("data", "adult_small.csv")
+
+    adult = pd.read_csv(data_path)
+    X_df = adult.loc[:, "age":"workclass"]
+    y_df = adult.loc[:, "class"]
+
+    X_train, X_test, y_train, y_test = train_test_split(
+        X_df, y_df, test_size=0.2
+    )
+
+    shadow = Foreshadow(
+        estimator=LogisticRegression(), problem_type=ProblemType.CLASSIFICATION
+    )
+
+    from foreshadow.concrete.internals.cleaners.customizable_base import (
+        CustomizableBaseCleaner,
+    )
+
+    def lowercase_row(row):
+        """Lowercase a row.
+
+        Args:
+            row: string of text
+
+        Returns:
+            transformed row.
+
+        """
+        # Without using the customizable base cleaner, we have to explain
+        # the meaning of the matched length. I don't know a good way to
+        # explain it clearly without diving into the internal details yet.
+        # return (row, 0) if row is None else (str(row).lower(), 1)
+
+        return row if row is None else str(row).lower()
+
+    class LowerCaseCleaner(CustomizableBaseCleaner):
+        def __init__(self):
+            super().__init__(transformation=lowercase_row)
+
+        def metric_score(self, X: pd.DataFrame) -> float:
+            """Calculate the matching metric score of the cleaner on this col.
+
+            In this method, you specify the condition on when to apply the
+            cleaner and calculate a confidence score between 0 and 1 where 1
+            means 100% certainty to apply the transformation.
+
+            Args:
+                X: a column as a dataframe.
+
+            Returns:
+                the confidence score.
+
+            """
+            # The user needs to know what cleaners are provided so that
+            # they don't create something duplicate or overlapping.
+            column_name = list(X.columns)[0]
+            if column_name == "workclass":
+                return 1
+            else:
+                return 0
+
+    shadow.register_customized_data_cleaner(data_cleaners=[LowerCaseCleaner])
+
+    workclass_values = list(X_train["workclass"].unique())
+    print(workclass_values)
+
+    X_train_cleaned = shadow.X_preparer.steps[1][1].fit_transform(X_train)
+
+    workclass_values_transformed = list(X_train_cleaned["workclass"].unique())
+    for value in workclass_values_transformed:
+        assert not any([c.isupper() for c in value])
+
+
+def test_set_processed_data_export_path():
+    from foreshadow.foreshadow import Foreshadow
+    from sklearn.linear_model import LogisticRegression
+    from foreshadow.utils import ConfigKey
+
+    shadow = Foreshadow(
+        estimator=LogisticRegression(), problem_type=ProblemType.CLASSIFICATION
+    )
+    processed_training_data_path = "datapath1.csv"
+    shadow.set_processed_data_export_path(
+        data_path=processed_training_data_path, is_train=True
+    )
+    assert (
+        shadow.X_preparer.cache_manager[AcceptedKey.CONFIG][
+            ConfigKey.PROCESSED_TRAINING_DATA_EXPORT_PATH
+        ]
+        == processed_training_data_path
+    )
+
+    processed_test_data_path = "datapath2.csv"
+    shadow.set_processed_data_export_path(
+        data_path=processed_test_data_path, is_train=False
+    )
+    assert (
+        shadow.X_preparer.cache_manager[AcceptedKey.CONFIG][
+            ConfigKey.PROCESSED_TEST_DATA_EXPORT_PATH
+        ]
+        == processed_test_data_path
+    )
+
+
+def test_foreshadow_titanic(tmpdir):
+    import pandas as pd
+
+    train_data = pd.read_csv(get_file_path("data", "titanic-train.csv"))
+    X_train_df = train_data.loc[:, "Pclass":"Embarked"]
+    y_train_df = train_data.loc[:, "Survived"]
+
+    X_train_df = X_train_df.drop(columns=["SibSp", "Parch", "Cabin"])
+
+    X_train, X_test, y_train, y_test = train_test_split(
+        X_train_df, y_train_df, test_size=0.2, random_state=42
+    )
+
+    from foreshadow.estimators import AutoEstimator
+
+    estimator = AutoEstimator(
+        problem_type=ProblemType.CLASSIFICATION,
+        auto="tpot",
+        estimator_kwargs={"max_time_mins": 1, "random_state": 42},
+    )
+
+    shadow = Foreshadow(
+        estimator=estimator, problem_type=ProblemType.CLASSIFICATION
+    )
+
+    shadow.override_intent(column_name="Name", intent=IntentType.TEXT)
+    shadow.fit(X_train, y_train)
+
+    score = shadow.score(X_test, y_test)
+    print(score)
+
+
+def test_text_classification_foreshadow():
+    import pandas as pd
+
+    categories = [
+        "alt.atheism",
+        "soc.religion.christian",
+        "comp.graphics",
+        "sci.med",
+    ]
+    from sklearn.datasets import fetch_20newsgroups
+
+    twenty_train = fetch_20newsgroups(
+        subset="train", categories=categories, shuffle=True, random_state=42
+    )
+    X_train = pd.DataFrame(
+        data=twenty_train.data,
+        columns=["text"],
+        index=list(range(len(twenty_train.data))),
+    )
+    y_train = pd.Series(
+        data=twenty_train.target,
+        name="category",
+        index=list(range(len(twenty_train.target))),
+    )
+
+    twenty_test = fetch_20newsgroups(
+        subset="test", categories=categories, shuffle=True, random_state=42
+    )
+    X_test = pd.DataFrame(data=twenty_test.data, columns=["text"])
+    y_test = pd.Series(data=twenty_test.target, name="category")
+
+    from foreshadow.estimators import AutoEstimator
+
+    estimator = AutoEstimator(
+        problem_type=ProblemType.CLASSIFICATION,
+        auto="tpot",
+        estimator_kwargs={"max_time_mins": 1, "random_state": 42},
+    )
+
+    shadow = Foreshadow(
+        estimator=estimator, problem_type=ProblemType.CLASSIFICATION
+    )
+
+    shadow.fit(X_train, y_train)
+
+    score = shadow.score(X_test, y_test)
+    print(score)  # this gives about 87.5%
+
+
+# def test_text_classification():
+#     # This is a test on text classification for pure Sklearn pipeline so we
+#     # can compare the result with foreshadow.
+#     # Using the NB estimator the performance is around 83.5%.
+#     categories = ['alt.atheism', 'soc.religion.christian',
+#                   'comp.graphics', 'sci.med']
+#     from sklearn.datasets import fetch_20newsgroups
+#     twenty_train = fetch_20newsgroups(subset='train',
+#                                       categories=categories, shuffle=True,
+#                                       random_state=42)
+#     print(twenty_train.target_names)
+#
+#     from sklearn.feature_extraction.text import CountVectorizer
+#     count_vect = CountVectorizer()
+#     X_train_counts = count_vect.fit_transform(twenty_train.data)
+#     print(X_train_counts.shape)
+#
+#     from sklearn.feature_extraction.text import TfidfTransformer
+#     tf_transformer = TfidfTransformer(use_idf=False).fit(X_train_counts)
+#     X_train_tf = tf_transformer.transform(X_train_counts)
+#     print(X_train_tf.shape)
+#
+#     tfidf_transformer = TfidfTransformer()
+#     X_train_tfidf = tfidf_transformer.fit_transform(X_train_counts)
+#     print(X_train_tfidf.shape)
+#
+#     from sklearn.naive_bayes import MultinomialNB
+#
+#     from sklearn.pipeline import Pipeline
+#     text_clf = Pipeline([
+#         ('vect', CountVectorizer()),
+#         ('tfidf', TfidfTransformer()),
+#         ('clf', MultinomialNB()),
+#     ])
+#
+#     text_clf.fit(twenty_train.data, twenty_train.target)
+#
+#     import numpy as np
+#     twenty_test = fetch_20newsgroups(subset='test',
+#                                      categories=categories, shuffle=True,
+#                                      random_state=42)
+#     docs_test = twenty_test.data
+#     predicted = text_clf.predict(docs_test)
+#     print(np.mean(predicted == twenty_test.target))
