@@ -1,6 +1,9 @@
 """Test the data_preparer.py file."""
 import pytest
 
+from foreshadow.concrete import NoTransform
+from foreshadow.smart import CategoricalEncoder
+from foreshadow.utils import ProblemType
 from foreshadow.utils.testing import get_file_path
 
 
@@ -36,7 +39,7 @@ def test_data_preparer_init(cleaner_kwargs, expected_error):
 
 
 @pytest.mark.parametrize("problem_type", [None, "Unknown"])
-def test_data_preparer_y_variable(problem_type):
+def test_data_preparer_y_variable_invalid_problem_type(problem_type):
     from foreshadow.preparer import DataPreparer
 
     with pytest.raises(ValueError) as e:
@@ -44,6 +47,20 @@ def test_data_preparer_y_variable(problem_type):
     assert "Invalid Problem Type" in str(e.value)
 
 
+@pytest.mark.parametrize(
+    "problem_type", [ProblemType.CLASSIFICATION, ProblemType.REGRESSION]
+)
+def test_data_preparer_y_variable(problem_type):
+    from foreshadow.preparer import DataPreparer
+
+    dp = DataPreparer(y_var=True, problem_type=problem_type)
+    assert len(dp.steps) == 1
+    if problem_type == ProblemType.REGRESSION:
+        assert isinstance(dp.steps[0][1], NoTransform)
+    else:
+        assert isinstance(dp.steps[0][1], CategoricalEncoder)
+
+
 @pytest.mark.parametrize("cleaner_kwargs", [({}), (None)])
 def test_data_preparer_fit(cleaner_kwargs):
     """Test fitting of DataPreparer after creation with kwargs.
@@ -86,61 +103,27 @@ def test_data_preparer_get_params(deep):
     assert "steps" in params
 
 
-def test_data_preparer_serialization_has_one_cache_manager():
-    """Test DataPreparer serialization after fitting. The serialized
-    object should contain only 1 cache_manager instance.
-
-    """
-    from foreshadow.preparer import DataPreparer
-    from foreshadow.cachemanager import CacheManager
-    import pandas as pd
-
-    boston_path = get_file_path("data", "boston_housing.csv")
-    data = pd.read_csv(boston_path)
-
-    cs = CacheManager()
-    dp = DataPreparer(cs)
-    dp.fit(data)
-
-    dp_serialized = dp.serialize(method="dict", deep=True)
-
-    key_name = "cache_manager"
-    assert key_name in dp_serialized
-    dp_serialized.pop(key_name)
-
-    def check_has_no_cache_manager(dat, target):
-        if isinstance(dat, dict):
-            matching_keys = [key for key in dat if key.endswith(target)]
-            assert len(matching_keys) == 0
-            for key in dat:
-                check_has_no_cache_manager(dat[key], target)
-        elif isinstance(dat, list):
-            for item in dat:
-                check_has_no_cache_manager(item, target)
-
-    check_has_no_cache_manager(dp_serialized, key_name)
-
-
-def test_data_preparer_deserialization(tmpdir):
-    from foreshadow.preparer import DataPreparer
-    from foreshadow.cachemanager import CacheManager
-    import pandas as pd
-
-    boston_path = get_file_path("data", "boston_housing.csv")
-    data = pd.read_csv(boston_path)
-
-    cs = CacheManager()
-    dp = DataPreparer(cs)
-
-    dp.fit(data)
-    data_transformed = dp.transform(data)
-    json_location = tmpdir.join("data_preparerer.json")
-    dp.to_json(json_location)
-
-    dp2 = DataPreparer.from_json(json_location)
-    dp2.fit(data)
-    data_transformed2 = dp2.transform(data)
-
-    from pandas.util.testing import assert_frame_equal
-
-    assert_frame_equal(data_transformed, data_transformed2)
+# def test_data_preparer_intent_resolving():
+#     from foreshadow.preparer import DataPreparer
+#     from foreshadow.cachemanager import CacheManager
+#     import pandas as pd
+#
+#     # from foreshadow.intents import IntentType
+#     # from foreshadow.utils import AcceptedKey, Override
+#
+#     data_path = get_file_path("data", "adult_small.csv")
+#     data = pd.read_csv(data_path)
+#
+#     cs = CacheManager()
+#     # cs[AcceptedKey.OVERRIDE][
+#     #     "_".join([Override.INTENT, 'age'])
+#     # ] = IntentType.CATEGORICAL
+#
+#     dp = DataPreparer(cs)
+#
+#     # data["crim"] = np.nan
+#
+#     dp.fit(data)
+#     res = dp.transform(data)
+#     print(cs["intent"])
+#     print(res)
