@@ -1,6 +1,28 @@
 """Test data_cleaner.py"""
+import pytest
 
 
+def test_data_cleaner_transform_before_fit():
+    import pandas as pd
+    from foreshadow.steps import CleanerMapper
+    from foreshadow.cachemanager import CacheManager
+
+    data = pd.DataFrame(
+        {"financials": ["$1.00", "$550.01", "$1234", "$12353.3345"]},
+        columns=["financials"],
+    )
+    cs = CacheManager()
+    dc = CleanerMapper(cache_manager=cs)
+
+    with pytest.raises(ValueError) as e:
+        dc.transform(data)
+
+    assert str(e.value) == "Cleaner has not been fitted yet."
+
+
+# TODO: This is no longer valid as we have separated data cleaner into
+#  flattener and cleaner
+@pytest.mark.skip("TODO: need to fix the flattener and cleaner issue.")
 def test_data_cleaner_fit():
     """Test basic fit call."""
     import pandas as pd
@@ -85,15 +107,13 @@ def test_json():
     import numpy as np
 
     data = pd.DataFrame(
-        {
-            "json": [
-                '{"date": "2019-04-11"}',
-                '{"financial": "$1.0"}',
-                '{"financial": "$1000.00"}',
-                '{"random": "asdf"}',
-            ]
-        },
-        columns=["json"],
+        [
+            ["2019-04-11", np.nan, np.nan],
+            [np.nan, "$1.0", np.nan],
+            [np.nan, "$1000.00", np.nan],
+            [np.nan, np.nan, "asdf"],
+        ],
+        columns=["json_date", "json_financial", "json_random"],
     )
     cs = CacheManager()
     dc = CleanerMapper(cache_manager=cs)
@@ -119,7 +139,7 @@ def test_json():
     )
 
 
-def test_drop():
+def test_drop_entire_data_frame():
     """Test drop called when expected to."""
     import pandas as pd
     from foreshadow.preparer import CleanerMapper
@@ -140,6 +160,24 @@ def test_drop():
     assert error_msg in str(excinfo.value)
 
 
+def test_drop_empty_columns():
+    """Test drop empty columns called when expected to."""
+    import pandas as pd
+    from foreshadow.preparer import CleanerMapper
+    from foreshadow.cachemanager import CacheManager
+
+    columns = ["financials", "nums"]
+    data = pd.DataFrame(
+        {"financials": ["", "", "", ""], "nums": [1, 2, 3, 4]}, columns=columns
+    )
+    cs = CacheManager()
+    dc = CleanerMapper(cache_manager=cs)
+
+    transformed_data = dc.fit_transform(data)
+    assert len(transformed_data.columns) == 1
+    assert list(transformed_data.columns)[0] == "nums"
+
+
 def test_numerical_input():
     """Test numerical input."""
     import numpy as np
