@@ -1,5 +1,10 @@
 import pytest
+from sklearn.feature_extraction.text import TfidfVectorizer
+from sklearn.pipeline import Pipeline
 
+from foreshadow.cachemanager import CacheManager
+from foreshadow.intents import IntentType
+from foreshadow.utils import AcceptedKey, TruncatedSVDWrapper
 from foreshadow.utils.testing import get_file_path
 
 
@@ -171,13 +176,12 @@ def test_smart_encoder_less_than_30_levels():
     from foreshadow.smart import CategoricalEncoder
     from foreshadow.concrete import OneHotEncoder
     from foreshadow.concrete import NaNFiller
-    from foreshadow.pipeline import SerializablePipeline
 
     np.random.seed(0)
     leq_30_random_data = np.random.choice(30, size=500)
     smart_coder = CategoricalEncoder()
     transformer = smart_coder.fit(leq_30_random_data).transformer
-    assert isinstance(transformer, SerializablePipeline)
+    assert isinstance(transformer, Pipeline)
     assert isinstance(transformer.steps[0][1], NaNFiller)
     assert isinstance(transformer.steps[1][1], OneHotEncoder)
     assert len(transformer.steps) == 2
@@ -189,16 +193,19 @@ def test_smart_encoder_more_than_30_levels():
     from foreshadow.smart import CategoricalEncoder
     from foreshadow.concrete import HashingEncoder
     from foreshadow.concrete import NaNFiller
-    from foreshadow.pipeline import SerializablePipeline
 
     np.random.seed(0)
     gt_30_random_data = np.random.choice(31, size=500)
+    gt_30_random_data = [item for item in gt_30_random_data.astype(str)]
+    gt_30_random_data[0] = np.nan
     smart_coder = CategoricalEncoder()
     transformer = smart_coder.fit(gt_30_random_data).transformer
-    assert isinstance(transformer, SerializablePipeline)
+    assert isinstance(transformer, Pipeline)
     assert isinstance(transformer.steps[0][1], NaNFiller)
     assert isinstance(transformer.steps[1][1], HashingEncoder)
     assert len(transformer.steps) == 2
+    res = transformer.transform(gt_30_random_data)
+    assert len(res.columns) == 30
 
 
 def test_smart_encoder_more_than_30_levels_that_reduces():
@@ -238,11 +245,11 @@ def test_smart_encoder_y_var():
 def test_smart_impute_simple_none():
     import numpy as np
     import pandas as pd
-    from foreshadow.smart import SimpleImputer
+    from foreshadow.smart import SimpleFillImputer
 
     heart_path = get_file_path("data", "heart-h.csv")
 
-    impute = SimpleImputer(threshold=0.05)
+    impute = SimpleFillImputer(threshold=0.05)
     df = pd.read_csv(heart_path)
 
     data = df[["chol"]]
@@ -256,12 +263,12 @@ def test_smart_impute_simple_none():
 def test_smart_impute_simple_mean():
     import numpy as np
     import pandas as pd
-    from foreshadow.smart import SimpleImputer
+    from foreshadow.smart import SimpleFillImputer
 
     heart_path = get_file_path("data", "heart-h.csv")
     heart_impute_path = get_file_path("data", "heart-h_impute_mean.csv")
 
-    impute = SimpleImputer()
+    impute = SimpleFillImputer()
     df = pd.read_csv(heart_path)
 
     data = df[["chol"]]
@@ -276,12 +283,12 @@ def test_smart_impute_simple_mean():
 def test_smart_impute_simple_median():
     import pandas as pd
     import numpy as np
-    from foreshadow.smart import SimpleImputer
+    from foreshadow.smart import SimpleFillImputer
 
     heart_path = get_file_path("data", "heart-h.csv")
     heart_impute_path = get_file_path("data", "heart-h_impute_median.csv")
 
-    impute = SimpleImputer()
+    impute = SimpleFillImputer()
     df = pd.read_csv(heart_path)
 
     data = df["chol"].values
@@ -367,13 +374,12 @@ def test_smart_encoder_delimmited():
     from foreshadow.smart import CategoricalEncoder
     from foreshadow.concrete import DummyEncoder
     from foreshadow.concrete import NaNFiller
-    from foreshadow.pipeline import SerializablePipeline
 
     data = pd.DataFrame({"test": ["a", "a,b,c", "a,b", "a,c"]})
     smart_coder = CategoricalEncoder()
     transformer = smart_coder.fit(data).transformer
 
-    assert isinstance(transformer, SerializablePipeline)
+    assert isinstance(transformer, Pipeline)
     assert isinstance(transformer.steps[0][1], NaNFiller)
     assert isinstance(transformer.steps[1][1], DummyEncoder)
     assert len(transformer.steps) == 2
@@ -384,14 +390,13 @@ def test_smart_encoder_more_than_30_levels_with_overwritten_cutoff():
     from foreshadow.smart import CategoricalEncoder
     from foreshadow.concrete import OneHotEncoder
     from foreshadow.concrete import NaNFiller
-    from foreshadow.pipeline import SerializablePipeline
 
     np.random.seed(0)
     gt_30_random_data = np.random.choice(31, size=500)
     smart_coder = CategoricalEncoder(unique_num_cutoff=35)
     transformer = smart_coder.fit(gt_30_random_data).transformer
 
-    assert isinstance(transformer, SerializablePipeline)
+    assert isinstance(transformer, Pipeline)
     assert isinstance(transformer.steps[0][1], NaNFiller)
     assert isinstance(transformer.steps[1][1], OneHotEncoder)
     assert len(transformer.steps) == 2
@@ -449,45 +454,61 @@ def test_smart_financial_cleaner_eu():
     assert np.all((out == expected) | (pd.isnull(out) == pd.isnull(expected)))
 
 
-@pytest.mark.skip("THIS IS IMPORTANT FIX")
-def test_smart_text():  # not sure why this is broken.
-    import numpy as np
+def test_smart_text():
     import pandas as pd
 
     from foreshadow.smart import TextEncoder
-    from foreshadow.concrete import FixedTfidfVectorizer
-    from foreshadow.concrete import HTMLRemover
-
-    X1 = pd.DataFrame(["abc", "def", "1321", "tester"])
-    tf1 = TextEncoder().fit(X1)
-
-    assert isinstance(tf1.transformer, FixedTfidfVectorizer)
-
-    X2 = pd.DataFrame(["<p> Hello </p>", "World", "<h1> Tag </h1>"])
-    tf2 = TextEncoder().fit(X2)
-
-    assert any(isinstance(tf, HTMLRemover) for n, tf in tf2.transformer.steps)
-    assert isinstance(tf2.transformer.steps[-1][1], FixedTfidfVectorizer)
-
-    assert TextEncoder().fit(pd.DataFrame([1, 2, 3, np.nan]))
-
-
-@pytest.mark.skip("NOT FIXED. TFIDF is broken.")
-def test_smart_neither():
-    import pandas as pd
-
-    from foreshadow.smart import NeitherProcessor
-    from foreshadow.concrete import FixedTfidfVectorizer
 
     X1 = pd.DataFrame(
-        [
-            "this is a test",
-            "I don't understand",
-            "who are you",
-            "what happened in this test",
-        ],
-        columns=["test_column"],
+        data={
+            "col1": ["abc", "def", "1321", "tester"],
+            "col2": ["okay", "This is a test", "whatup", "gg"],
+        }
     )
-    tf1 = NeitherProcessor().fit(X1)
 
-    assert isinstance(tf1.transformer, FixedTfidfVectorizer)
+    manager = CacheManager()
+    manager[AcceptedKey.INTENT, "col1"] = IntentType.TEXT
+
+    encoder1 = TextEncoder(n_components=3)
+    tf1 = encoder1.fit(X1)
+    X1_transformed = tf1.transform(X=X1)
+    print()
+    print(X1_transformed)
+
+    assert isinstance(tf1.transformer.steps[-2][1], TfidfVectorizer)
+    assert isinstance(tf1.transformer.steps[-1][1], TruncatedSVDWrapper)
+
+    # X2 = pd.DataFrame(
+    #     data=["<p> Hello </p>", "World", "<h1> Tag </h1>", 123],
+    #     columns=["col2"],
+    # )
+    #
+    # encoder2 = TextEncoder()
+    # tf2 = encoder2.fit(X2)
+    # X2_transformed = tf2.transform(X=X2)
+    # print(X2_transformed)
+    #
+    # assert any(isinstance(tf, ToString) for n, tf in tf2.transformer.steps)
+    # assert any(isinstance(tf, HTMLRemover) for n,
+    # tf in tf2.transformer.steps)
+    # assert isinstance(tf2.transformer.steps[0][1], ToString)
+    # assert isinstance(tf2.transformer.steps[1][1], HTMLRemover)
+    # assert isinstance(tf2.transformer.steps[2][1], DataSeriesSelector)
+    # assert isinstance(tf2.transformer.steps[3][1], TfidfVectorizer)
+
+
+def test_smart_text_wrong_intent():
+    import pandas as pd
+
+    from foreshadow.smart import TextEncoder
+
+    X1 = pd.DataFrame(data=["1", "4", "a", "a"], columns=["col1"])
+
+    manager = CacheManager()
+    manager[AcceptedKey.INTENT, "col1"] = IntentType.TEXT
+
+    encoder1 = TextEncoder(cache_manager=manager)
+
+    with pytest.raises(ValueError) as e:
+        encoder1.fit(X1)
+        assert "empty vocabulary" in str(e)
