@@ -4,12 +4,16 @@
 class DefaultConfig:
     """Constants for default configurations."""
 
-    PROCESSED_DATA_EXPORT_PATH = "processed_data.csv"
+    PROCESSED_TRAINING_DATA_EXPORT_PATH = "processed_training_data.csv"
+    PROCESSED_TEST_DATA_EXPORT_PATH = "processed_test_data.csv"
     ENABLE_SAMPLING = True
     SAMPLING_DATASET_SIZE_THRESHOLD = 10000
     SAMPLING_WITH_REPLACEMENT = False
     SAMPLING_FRACTION = 0.2
     N_JOBS = 1
+    # It is unclear what is the best value. The default value in Sklearn is 2,
+    # which may not be enough.
+    N_COMPONENTS_SVD = 20
 
 
 class ProblemType:
@@ -36,7 +40,9 @@ class ConfigKey:
     SAMPLING_WITH_REPLACEMENT = "with_replacement"
     SAMPLING_FRACTION = "sampling_fraction"
     N_JOBS = "n_jobs"
-    PROCESSED_DATA_EXPORT_PATH = "processed_data_export_path"
+    PROCESSED_TRAINING_DATA_EXPORT_PATH = "processed_training_data_export_path"
+    PROCESSED_TEST_DATA_EXPORT_PATH = "processed_test_data_export_path"
+    CUSTOMIZED_CLEANERS = "customized_cleaners"
 
 
 class AcceptedKey:
@@ -48,6 +54,7 @@ class AcceptedKey:
     GRAPH = "graph"
     OVERRIDE = "override"
     CONFIG = "config"
+    CUSTOMIZED_TRANSFORMERS = "customized_transformers"
 
 
 class Constant:
